@@ -13,23 +13,20 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import socket
+import sys
+from urllib import parse as urlparse
+
 import eventlet
 # oslo_messaging/notify/listener.py documents that monkeypatching is required
 eventlet.monkey_patch()
-
-from six.moves.urllib import parse
-import socket
-import sys
-
-from ironicclient import client
-import ironicclient.common.apiclient.exceptions as ironic_exc
-from keystoneauth1 import loading
 from neutron.agent import rpc as agent_rpc
 from neutron.common import config as common_config
 from neutron.conf.agent import common as agent_config
 from neutron_lib.agent import topics
 from neutron_lib import constants as n_const
 from neutron_lib import context
+from openstack import exceptions as sdk_exc
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
@@ -40,83 +37,27 @@ from oslo_utils import uuidutils
 from tooz import hashring
 
 from networking_baremetal import constants
+from networking_baremetal import ironic_client
 
-DEFAULT_IRONIC_API_VERSION = 'latest'
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
-IRONIC_SESSION = None
-IRONIC_GROUP = 'ironic'
-
-
-IRONIC_OPTS = [
-    cfg.StrOpt('os_region',
-               help=_('Keystone region used to get Ironic endpoints.')),
-    cfg.StrOpt('auth_strategy',
-               default='keystone',
-               choices=('keystone', 'noauth'),
-               help=_('Method to use for authentication: noauth or '
-                      'keystone.')),
-    cfg.StrOpt('ironic_url',
-               default='http://localhost:6385/',
-               help=_('Ironic API URL, used to set Ironic API URL when '
-                      'auth_strategy option is noauth to work with standalone '
-                      'Ironic without keystone.')),
-    cfg.IntOpt('retry_interval',
-               default=2,
-               help=_('Interval between retries in case of conflict error '
-                      '(HTTP 409).')),
-    cfg.IntOpt('max_retries',
-               default=30,
-               help=_('Maximum number of retries in case of conflict error '
-                      '(HTTP 409).')),
-]
-
-CONF.register_opts(IRONIC_OPTS, group=IRONIC_GROUP)
 CONF.import_group('AGENT', 'neutron.plugins.ml2.drivers.agent.config')
 
 
 def list_opts():
-    return [(IRONIC_GROUP, IRONIC_OPTS +
-             loading.get_session_conf_options() +
-             loading.get_auth_plugin_conf_options('v3password')),
-            ('agent', agent_config.AGENT_STATE_OPTS)]
-
-
-def get_session(group):
-    loading.register_session_conf_options(CONF, group)
-    loading.register_auth_conf_options(CONF, group)
-    auth = loading.load_auth_from_conf_options(CONF, group)
-    session = loading.load_session_from_conf_options(
-        CONF, group, auth=auth)
-    return session
-
-
-def get_client(api_version=DEFAULT_IRONIC_API_VERSION):
-    """Get Ironic client instance."""
-    # NOTE: To support standalone ironic without keystone
-    if CONF.ironic.auth_strategy == 'noauth':
-        args = {'token': 'noauth',
-                'endpoint': CONF.ironic.ironic_url}
-    else:
-        global IRONIC_SESSION
-        if not IRONIC_SESSION:
-            IRONIC_SESSION = get_session(IRONIC_GROUP)
-        args = {'session': IRONIC_SESSION,
-                'region_name': CONF.ironic.os_region}
-    args['os_ironic_api_version'] = api_version
-    args['max_retries'] = CONF.ironic.max_retries
-    args['retry_interval'] = CONF.ironic.retry_interval
-    return client.Client(1, **args)
+    return [('agent', agent_config.AGENT_STATE_OPTS)]
 
 
 def _get_notification_transport_url():
-    url = parse.urlparse(CONF.transport_url)
-    if CONF.oslo_messaging_rabbit.amqp_auto_delete is False:
-        q = parse.parse_qs(url.query)
+    url = urlparse.urlparse(CONF.transport_url)
+    if (CONF.oslo_messaging_rabbit.amqp_auto_delete is False
+            and not getattr(CONF.oslo_messaging_rabbit, 'rabbit_quorum_queue',
+                            None)):
+        q = urlparse.parse_qs(url.query)
         q.update({'amqp_auto_delete': ['true']})
-        query = parse.urlencode({k: v[0] for k, v in q.items()})
+        query = urlparse.urlencode({k: v[0] for k, v in q.items()})
         url = url._replace(query=query)
-    return parse.urlunparse(url)
+    return urlparse.urlunparse(url)
 
 
 def _set_up_notifier(transport, uuid):
@@ -155,8 +96,8 @@ class HashRingMemberManagerNotificationEndpoint(object):
                 self.hashring.add_node(payload['id'])
                 self.members.append(payload)
             except Exception:
-                LOG.exception(
-                    'Failed to add member %s to hash ring!' % payload['id'])
+                LOG.exception('Failed to add member %s to hash ring!',
+                              payload['id'])
         else:
             for member in self.members:
                 if payload['id'] == member['id']:
@@ -172,8 +113,8 @@ class HashRingMemberManagerNotificationEndpoint(object):
                     self.hashring.remove_node(member['id'])
                     self.members.remove(member)
                 except Exception:
-                    LOG.exception('Failed to remove member %s from hash ring!'
-                                  % member['id'])
+                    LOG.exception('Failed to remove member %s from hash ring!',
+                                  member['id'])
 
         return oslo_messaging.NotificationResult.HANDLED
 
@@ -205,7 +146,7 @@ class BaremetalNeutronAgent(service.ServiceBase):
         self.member_manager = HashRingMemberManagerNotificationEndpoint()
 
         self.state_rpc = agent_rpc.PluginReportStateAPI(topics.REPORTS)
-        self.ironic_client = get_client()
+        self.ironic_client = ironic_client.get_client()
         self.reported_nodes = {}
         LOG.info('Agent networking-baremetal initialized.')
 
@@ -225,11 +166,19 @@ class BaremetalNeutronAgent(service.ServiceBase):
         LOG.info('Stopping agent networking-baremetal.')
         self.heartbeat.stop()
         self.notify_agents.stop()
+        self.listener.stop()
+        self.pool_listener.stop()
+        self.listener.wait()
+        self.pool_listener.wait()
 
     def reset(self):
         LOG.info('Resetting agent networking-baremetal.')
         self.heartbeat.stop()
         self.notify_agents.stop()
+        self.listener.stop()
+        self.pool_listener.stop()
+        self.listener.wait()
+        self.pool_listener.wait()
 
     def wait(self):
         pass
@@ -243,7 +192,7 @@ class BaremetalNeutronAgent(service.ServiceBase):
                  'host': self.agent_host,
                  'timestamp': timeutils.utcnow_ts()})
         except Exception:
-            LOG.exception('Failed to send hash ring membership hearbeat!')
+            LOG.exception('Failed to send hash ring membership heartbeat!')
 
     def get_template_node_state(self, node_uuid):
         return {
@@ -257,40 +206,28 @@ class BaremetalNeutronAgent(service.ServiceBase):
             'start_flag': False,
             'agent_type': constants.BAREMETAL_AGENT_TYPE}
 
-    def _get_ironic_ports(self):
-        ironic_ports = []
-        try:
-            ironic_ports = self.ironic_client.port.list(detail=True)
-        except ironic_exc.UnsupportedVersion:
-            LOG.exception("Failed to get ironic port data! Ironic Client is "
-                          "using unsupported version of the API. State "
-                          "reporting for agent will be disabled.")
-            self.heartbeat.stop()
-        except (ironic_exc.AuthPluginOptionsMissing,
-                ironic_exc.AuthSystemNotFound):
-            LOG.exception("Failed to get ironic port data! Ironic Client "
-                          "autorization failure. State reporting for agent "
-                          "will be disabled.")
-            self.heartbeat.stop()
-        except Exception:
-            LOG.exception("Failed to get ironic port data!")
-
-        return ironic_ports
-
     def _report_state(self):
         node_states = {}
-        ironic_ports = self._get_ironic_ports()
-        if not ironic_ports:
+        ironic_ports = self.ironic_client.ports(details=True)
+        # NOTE: the above calls returns a generator, so we need to handle
+        # exceptions that happen just before the first loop iteration, when
+        # the actual request to ironic happens
+        try:
+            for port in ironic_ports:
+                node = port.node_id
+                if (self.agent_id not in
+                        self.member_manager.hashring[node.encode('utf-8')]):
+                    continue
+                template_node_state = self.get_template_node_state(node)
+                node_states.setdefault(node, template_node_state)
+                mapping = node_states[
+                    node]["configurations"]["bridge_mappings"]
+                if port.physical_network is not None:
+                    mapping[port.physical_network] = "yes"
+        except sdk_exc.OpenStackCloudException:
+            LOG.exception("Failed to get ironic ports data! "
+                          "Not reporting state.")
             return
-        for port in ironic_ports:
-            node = port.node_uuid
-            if (self.agent_id not in
-                    self.member_manager.hashring[node.encode('utf-8')]):
-                continue
-            node_states.setdefault(node, self.get_template_node_state(node))
-            mapping = node_states[node]["configurations"]["bridge_mappings"]
-            if port.physical_network is not None:
-                mapping[port.physical_network] = "yes"
 
         for state in node_states.values():
             # If the node was not previously reported with current
@@ -320,7 +257,20 @@ class BaremetalNeutronAgent(service.ServiceBase):
                 {state['host']: state['configurations']})
 
 
+def _unregiser_deprecated_opts():
+    CONF.reset()
+    CONF.unregister_opts(
+        [CONF._groups[ironic_client.IRONIC_GROUP]._opts[opt]['opt']
+         for opt in ironic_client._deprecated_opts],
+        group=ironic_client.IRONIC_GROUP)
+
+
 def main():
+    common_config.register_common_config_options()
+    # TODO(hjensas): Imports from neutron in ironic_neutron_agent registers the
+    # client options. We need to unregister the options we are deprecating
+    # first to avoid DuplicateOptError. Remove this when dropping deprecations.
+    _unregiser_deprecated_opts()
     common_config.init(sys.argv[1:])
     common_config.setup_logging()
     agent = BaremetalNeutronAgent()
